function love.draw()
	if love.mouse.isDown("l") then
		love.graphics.setBlendMode("additive")
		love.graphics.setColor(255, 255, 255, 96)
		paintAtMouse()
	elseif love.mouse.isDown("r") then
		love.graphics.setBlendMode("subtractive")
		love.graphics.setColor(255, 255, 255, 96)
		paintAtMouse()
	end
	love.graphics.setBlendMode("alpha")
	love.graphics.setColor(255, 255, 255, 255)
	drawShadowBuffer()
	
	love.graphics.setShader(drawVolumes)
	love.graphics.draw(terrain, 0, 0, 0, 1, 1)
	love.graphics.setShader()
	love.graphics.draw(shadowBuffer, 0, 0, 0, 0.2, 0.2)
	love.graphics.circle("fill", lightSource[1], lightSource[2], 5, 16)
	
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.setLineWidth(1)
	love.graphics.setLineStyle("smooth")	
	love.graphics.circle("line", mouseX, mouseY, brushRadius, 64)
end

function paintAtMouse()
	local falloff = 1
	love.graphics.setCanvas(terrain)
	love.graphics.setLineWidth(brushRadius * 2)
	local dist = math.sqrt((mouseX - prevMouseX)*(mouseX - prevMouseX) + (mouseY - prevMouseY)*(mouseY - prevMouseY))
	dist = math.floor(dist)
	local stepX = (mouseX - prevMouseX) / dist
	local stepY = (mouseY - prevMouseY) / dist
	--paint circles with constant intervals
	for i= 1, dist, 5 do
		local x = prevMouseX + i * stepX
		local y = prevMouseY + i * stepY
		love.graphics.setShader(paintCircle)
		paintCircle:send("falloff", falloff)
		paintCircle:send("radius", brushRadius)
		paintCircle:send("circleCentre", {x, love.graphics.getHeight() - y})
		love.graphics.circle("fill", x, y, brushRadius, 16 + math.floor(brushRadius / 10))
	end
	love.graphics.setCanvas()
	love.graphics.setShader()
end

function drawShadowBuffer()
	love.graphics.setCanvas(shadowBuffer)
	love.graphics.clear()
	love.graphics.setShader(polarCoordinateMap)
	love.graphics.draw(img, 0, 0, 0, shadowBuffer:getWidth(), shadowBuffer:getHeight())
	love.graphics.setCanvas(tmpBuffer)
	love.graphics.setShader(accumulateShadow)
	love.graphics.draw(img, 0, 0, 0, shadowBuffer:getWidth(), shadowBuffer:getHeight())
	love.graphics.setCanvas(shadowBuffer)
	love.graphics.setShader()
	love.graphics.draw(tmpBuffer)
	love.graphics.setCanvas()
end

function love.update(dt)
	prevMouseX = mouseX
	prevMouseY = mouseY
	mouseX = love.mouse.getX()
	mouseY = love.mouse.getY()
	arrowKeysInput()
	polarCoordinateMap:send('lightSource', {lightSource[1], love.graphics.getHeight() - lightSource[2], lightSource[3]})
	drawVolumes:send('lightSource', {lightSource[1], love.graphics.getHeight() - lightSource[2], lightSource[3]})
	love.window.setTitle( love.timer.getFPS().." FPS")
end
	
function love.mousepressed(x, y, button)
   if button == "wu" then
      brushRadius = brushRadius * 1.2
   end
   if button == "wd" then
      brushRadius = brushRadius / 1.2
   end
end

function arrowKeysInput()
	if love.keyboard.isDown( "up" ) then
		lightSource[2] = lightSource[2] - 2
	end
	if love.keyboard.isDown( "down" ) then
		lightSource[2] = lightSource[2] + 2
	end
	if love.keyboard.isDown( "left" ) then
		lightSource[1] = lightSource[1] - 2
	end
	if love.keyboard.isDown( "right" ) then
		lightSource[1] = lightSource[1] + 2
	end
end

function love.load()
	img = love.graphics.newImage(love.image.newImageData(1,1)) --dummy image, for use with shaders
	terrain = love.graphics.newCanvas(winwidth,winheight)
	mouseX = love.mouse.getX()
	mouseY = love.mouse.getY()
	brushRadius = 8
	groundPoints = {}
	lightSource = {love.graphics.getWidth()/2, love.graphics.getHeight()/2, 500} --x, y, range
	shadowBuffer = love.graphics.newCanvas(1024, lightSource[3])
	tmpBuffer = love.graphics.newCanvas(shadowBuffer:getWidth(), shadowBuffer:getHeight())
	groundTex1 = love.graphics.newImage("textures/test_circles.png")
	groundTex1:setWrap("repeat", "repeat")
	
	paintCircle = love.graphics.newShader [[
		extern vec2 circleCentre;
		extern float radius;
		extern float falloff;
        vec4 effect( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords )
        {
			float alpha = pow(1 - distance(screen_coords, circleCentre) / radius, falloff);
			color.a *= alpha;
            return color;
        }
    ]]
	
	polarCoordinateMap = love.graphics.newShader [[
		extern Image cartesianSource;
		extern vec2 sourceResolution;
		extern vec3 lightSource; //x, y, depth
        vec4 effect( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords )
        {
			float angle = 6.28318530718 * texture_coords.x;
			float depth = texture_coords.y * lightSource.z;
			vec2 cartesianCoords = lightSource.xy + vec2(depth*cos(angle), depth*sin(angle));
            return Texel(cartesianSource, cartesianCoords / sourceResolution);
        }
    ]]
	polarCoordinateMap:send('cartesianSource', terrain)
	polarCoordinateMap:send('sourceResolution', {love.graphics.getWidth(), love.graphics.getHeight()})
	
	accumulateShadow = love.graphics.newShader [[
		extern Image source;
		extern vec2 resolution;
		vec4 effect( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords )
        {
			color = vec4(0, 0, 0, 0);
			for(float i = screen_coords.y; i < resolution.y; i++){
				vec4 opacity = Texel(source, vec2(screen_coords.x, i) / resolution);
				color += 0.1 * opacity * opacity;
				if(color.r >= 1){
					break;
				}
			}
			return color;
        }
	]]
	accumulateShadow:send('source', shadowBuffer)
	accumulateShadow:send('resolution', {shadowBuffer:getWidth(), shadowBuffer:getHeight()})
	
	drawVolumes = love.graphics.newShader [[
		extern Image shadowBuffer;
		extern vec2 shadowBufferResolution;
		extern vec2 resolution;
		extern vec3 lightSource; //x, y, depth
        vec4 effect( vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords )
        {
			vec2 offset = screen_coords - lightSource.xy;
			float dist = length(offset);
			float angle = atan(-offset.y, -offset.x);
			angle += 3.141592;
			vec4 shadow = Texel(shadowBuffer, vec2(angle/6.28318530718, 1 - dist/shadowBufferResolution.y));
			vec4 light = vec4(1.0, 0.95, 0.7, 1.0) * (1 - dist/lightSource.z);
			vec4 ambient = vec4(0.1, 0.1, 0.2, 1.0);
			light = light * (1-shadow);
			light += ambient;
			color = Texel(texture, texture_coords);
            return color * light;
        }
    ]]
	drawVolumes:send('shadowBuffer', shadowBuffer)
	drawVolumes:send('shadowBufferResolution', {shadowBuffer:getWidth(), shadowBuffer:getHeight()})
	--drawVolumes:send('resolution', {love.graphics.getWidth(), love.graphics.getHeight()})
end
